
/*
 ******************************************
 **    © by Widgetschmie.de 2005-2008    **
 ******************************************
 */

var channelsLoaded = false;					// true if the channels have been loaded from prefs. Stays true during the runtime of Fernsehen.
var channelsAvailable = false;				// true if we currently hold _all_ channels in memory

var channelGroup1 = new Array();			// holds the IDs (!!!) of the assigned channels
var channelGroup2 = new Array();

var allChannelParents = new Array();		// original parents of the channels. Only filled if channelsAvailable is true
var allChannels = new Array();				// holds all channels IF channelsAvailable IS TRUE

var availableChannelGroups = new Array('channelGroup1', 'channelGroup2');		// Groups we may assign channels to
var availableChannels = new Object();											// holds only the channels which are assigned to either group of availableChannelGroups

var channelListVersion;
var newChannelListVersion;
var newChannelDownloadURL;




// loads all channels from the XML into the array allChannels
function loadAllChannels(and_show_list) {
	var path = FernsehPlugin.channelsPathForBundlePath(LANG + ".lproj/channels.xml", LANG);
	var ajax = new simpleAjaxRequest(path, allChannelsFetched, false, and_show_list);
	if(!ajax)
		return false;
}

// gets the channels, loads the channel assignments from the prefs and loads them into the menus in the widget
function allChannelsFetched(xml, and_show_list) {
	if(!xml) {
		return false;
	}
	
	// ****
	// create the channelParentObj and channelObj - objects
	var root = xml.documentElement;
	var elements = getElementNodes(root);
	if(elements && elements.length > 0) {
		for(var i = 0; i < elements.length; i++) {
			new channelParentObj(elements[i]);
		}
	}
	
	channelListVersion = (getChildNodeNamed(xml, "channellistversion")).firstChild.data;
	//alert("channellistversion "+channelListVersion);
	
	channelsAvailable = true;
	
	// ****
	// load from Prefs (we may forget about allChannels right after we're done loading)
	if(!channelsLoaded) {
		
		channelGroupFromPrefs("channelGroup1");
		channelGroupFromPrefs("channelGroup2");
		
		// make sure at least channelGroup1 is filled with a couple of default values for the first start
		var numAssignedMain = channelGroup1 ? channelGroup1.length : 0;
		if(0 == numAssignedMain) {
			///alert("no assigned channels, loading default channels, allChannels.length: " + allChannels.length + ", LOCALE: " + LOCALE);
			
			// loop all channels and load default channels
			for(var c in allChannels) {
				if(inArray(LOCALE, allChannels[c].default_for_locale)) {
					addChannelToGroup(c, "channelGroup1");
				}
			}
			
			saveChannelGroupings();
		}
		
		channelsLoaded = true;
		
		updateChannelsInPlugin();

		
		// clean unused channels from memory and load the assigns for the tuner support
		if(!and_show_list) {
			cleanUnusedChannels();
		}
		assignTunerChannels();
	}
	
	// show the channel list
	if(and_show_list) {
		toggleChannelSourceList();
	}
}


// cleans the unused channels from memony
function cleanUnusedChannels() {
	cleanNode('channelParentSourceArea');
	cleanNode('channelSourceArea');
	
	allChannelParents = new Array();
	allChannels = new Array();
	
	channelsAvailable = false;
}


// loads the channels from the Prefs
function channelGroupFromPrefs(group) {
	var parent = getObj(group);
	if(!parent || !channelsAvailable) {
		alert("Parent '" + group + "' not found or the channels are not loaded");
		return;
	}
	
	if(window.widget && widget.preferenceForKey(group)) {
		cleanNode(parent);
		var grouparr = widget.preferenceForKey(group).split("|");
		
		for(i = 0; i < grouparr.length; i++) {
			var channel_id = grouparr[i];
			if(allChannels[channel_id]) {
				addChannelToGroup(channel_id, group);
			}
		}
		
		channelsLoaded = true;
	}
}

// use this to assign a channel to one of our two main channel groups
function addChannelToGroup(channel_id, group) {
	if(!channelsAvailable) {
		alert("Channels are not loaded [3]");
		return;
	}
	
	if(allChannels[channel_id]) {
		allChannels[channel_id].setGroup(group);
		window[group].push(channel_id);
	}
}

// saves the channel groups to the prefs
function saveChannelGroupings() {
	if(!channelsAvailable)
		return;
	
	// hide channel source list if it is still shown
	if(channelSourceListShown) {
		toggleChannelSourceList();
	}
	
	// save new channel-order for all groups
	for(var i = 0; i < availableChannelGroups.length; i++) {
		var groupname = availableChannelGroups[i];
		
		var nodes = getElementNodes(getObj(groupname));
		var prefstring = '';
		window[groupname] = new Array();			// this is either the channelGroup1 or channelGroup2 array
		
		for(var m = 0; m < nodes.length; m++) {
			if("divDrag_item" == nodes[m].getAttribute("class")) {
				var channel_id = nodes[m].lang;
				if(allChannels[channel_id]) {
					prefstring += channel_id + "|";
					addChannelToGroup(channel_id, groupname);
				}
			}
		}
		
		widget.setPreferenceForKey(prefstring, groupname);
	}
	
	// clean up
	needsToCheckCache = true;
	cacheAhead();
	cleanUnusedChannels();
	updateChannelsInPlugin();
}

function updateChannelsInPlugin()
{
	var channelNames = new Array();
	var channelIDs = new Array();

	for (var i = 0; i < channelGroup1.length; i++)
	{
		var key = channelGroup1[i];
		channelIDs.push(key);
		channelNames.push((availableChannels[key]).getName());
	}
	for (var i = 0; i < channelGroup2.length; i++)
	{
		var key = channelGroup2[i];
		channelIDs.push(key);
		channelNames.push((availableChannels[key]).getName());
	}
	FernsehPlugin.setUsedChannels(channelIDs.join("\n"), channelNames.join("\n"));
	setChannelNamesInPlugin();
}


function setChannelNamesInPlugin()
{
	for (channel in availableChannels)
	{
		FernsehPlugin.setRealChannelName(availableChannels[channel].getName(), channel);
		FernsehPlugin.setTunerChannelName(availableChannels[channel].getTuner(), channel);
	}
}


// adds the channel-parents to a given div
function addChannelParentsToSelector(id) {
	var parent = getObj(id);
	if(!channelsAvailable) {
		alert("Channels are not loaded [1]");
		return;
	}
	
	if(parent && allChannelParents) {
		cleanNode(parent);
		
		// first, a searchfield
		var search = document.createElement('div');
		search.setAttribute('class', "click_list_channel_parent");
		search.setAttribute('onclick', "searchChannels(getObj('string_channelSearchField').value, this, 'channelSourceArea')");
		search.innerHTML = "<img src='Images/moreRight.png' alt='»' class='moreRight' /><input id=\"string_channelSearchField\" type=\"search\" incremental=\"incremental\" size=\"10\" placeholder=\"" + locStr("Suche Sender") + "\" onsearch=\"searchChannels(this.value, this.parentNode, 'channelSourceArea')\" />";
		parent.appendChild(search);
		
		// add the parents as clickable items
		for(var i = 0; i < allChannelParents.length; i++) {
			var group = null;
			var channels = allChannelParents[i].getChannels().length;
			
			group = document.createElement('div');
			group.setAttribute('class', "click_list_channel_parent");
			group.setAttribute('onclick', "showChannelsOfParentIn(this, '" + allChannelParents[i].getID() + "', 'channelSourceArea')");
			group.innerHTML = "<img src='Images/moreRight.png' alt='»' class='moreRight' />" + allChannelParents[i].getName() + " (" + channels + ")";
			
			if(group && group.childNodes && group.childNodes.length > 0) {
				parent.appendChild(group);
			}
		}
	}
	else {
		alert("addChannelParentsToSelector(): missing parent or addChannelParentsToSelector");
	}
}


// loads the channels of a given group as draggable objects (divDrag) into object with obj_id
var oldActiveChannelParent = null;
function showChannelsOfParentIn(parent_div, parent_id, obj_id) {
	if(!channelsAvailable) {
		alert("Channels are not loaded [2]");
		return;
	}
	
	parent_div = getObj(parent_div);
	var obj = getObj(obj_id);
	if(parent_div && parent_id && obj) {
		if(oldActiveChannelParent && (oldActiveChannelParent != parent_div)) {
			oldActiveChannelParent.setAttribute('class', "click_list_channel_parent");
		}
		parent_div.setAttribute('class', "click_list_channel_parent click_list_channel_parent_active");
		oldActiveChannelParent = parent_div;
		
		// clean old channels
		cleanNode(obj);
		
		// add channels
		var parent = getChannelParentByID(parent_id);
		var channels = parent.getChannels();
		for(var i = 0; i < channels.length; i++) {
			obj.appendChild(channels[i].getDraggableDiv());
		}
	}
	else {
		alert("showChannelsOfParentIn(): no parent or no obj");
	}
}


// searches channel names and inserts them as draggable div
function searchChannels(string, parent, target) {
	if(!channelsAvailable) {
		alert("Channels are not loaded [3]");
		return;
	}
	
	parent = getObj(parent);
	target = getObj(target);
	if(parent && target) {
		if(oldActiveChannelParent && (oldActiveChannelParent != target)) {
			oldActiveChannelParent.setAttribute('class', "click_list_channel_parent");
		}
		parent.setAttribute('class', "click_list_channel_parent click_list_channel_parent_active");
		oldActiveChannelParent = parent;
		
		// clean old channels
		cleanNode(target);
		
		// search for channels
		if(string && (string.length > 0)) {
			for(var id in allChannels) {
				if(allChannels[id] && allChannels[id].nameMatches(string)) {
					target.appendChild(allChannels[id].getDraggableDiv());
				}
			}
		}
	}
	else {
		alert("searchChannels(): no parent or no target");
	}
}


// adds the ASSIGNED channels to a given select menu or the pref-channel-lists
function addAssignedChannelsToSelector(id, display_option) {
	if(!channelsLoaded) {
		alert("Channels have not yet been loaded from Prefs. Cannot assign to " + id);
		return;
	}
	
	var parent = getObj(id);
	if(parent && availableChannelGroups) {
		var shownChannels = new Array();
		
		// empty parent
		cleanNode(parent);
		
		// add the groups themselves to the list for filter settings
		if('filter_list' == display_option) {
			var grp = document.createElement('option');
			grp.setAttribute('value', 0);
			grp.innerHTML = locStr('in allen Sendern');
			parent.appendChild(grp);
			var hr = has1043Features() ? document.createElement('hr') : document.createElement('option');
			parent.appendChild(hr);
			
			for(var i = 0; i < availableChannelGroups.length; i++) {
				var groupname = availableChannelGroups[i];
				var grp = document.createElement('option');
				grp.setAttribute('value', groupname);
				grp.innerHTML = locStr(groupname);
				parent.appendChild(grp);
			}
			
			hr = has1043Features() ? document.createElement('hr') : document.createElement('option');
			parent.appendChild(hr);
		}
		
		
		// ****
		// add the groups
		for(var i = 0; i < availableChannelGroups.length; i++) {
			var groupname = availableChannelGroups[i];
			var group = null;
			switch(display_option) {
				case 'tuner_senderlist':		group = document.createElement('div');
								var header = document.createElement('div');
								header.setAttribute('class', "tunerChannelGroupHeader");
								header.innerHTML = locStr(groupname);
								group.appendChild(header);
								break;
				case 'filter_list':
				default:		group = document.createElement('optgroup');
								group.setAttribute('label', locStr(groupname));
								break;
			}
			
			// the channels
			var channels = window[groupname];
			for(var c = 0; c < channels.length; c++) {
				if(inArray(channels[c], shownChannels)) {			// do show channels only once
					continue;
				}
				shownChannels.push(channels[c]);
				
				var chan = null;
				var channel = availableChannels[channels[c]];
				if(!channel) {
					alert("Channel " + channels[c] + " is no longer available");
					continue;
				}
				
				switch(display_option) {
					case 'tuner_senderlist':	// ARD: <select class="assign_sender" name="tuner_assign_ARD" id="tuner_assign_ARD"></select><br />
									var div = document.createElement('div');
									var id = channel.getID();
									var tc = channel.getTuner();
									var tc_index = indexOfValueInArray(tunerChannels, tc) + (has1043Features() ? 2 : 4);
									var tcn = ((tunerUsed == tunerChannelsAssignedFor) && tc) ? tc : locStr("zuweisen...");
									var html = channel.getName() + ': <select lang="' + id +'" class="assign_sender" id="tuner_assign_' + id +'" onmousedown="createTunerAssignMenu(this)"><option lang="' + tc_index + '" value="' + tc + '">' + tcn + '</option></select>';
									div.innerHTML = html;
									group.appendChild(div);
									break;
					case 'filter_list':
					default:		chan = document.createElement('option');
									chan.setAttribute('value', channel.getID());
									chan.innerHTML = channel.getName();
									group.appendChild(chan);
									break;
				}
			}
			
			if(group && group.childNodes && group.childNodes.length > 0) {
				parent.appendChild(group);
			}
		}
	}
}



// removes a channel from a group
function animateRemoveChannelFromGroup(obj) {
	var parent = obj.parentNode;
	if(!parent)
		return;
	
	if(allChannels[obj.lang] || availableChannels[obj.lang]) {
		obj.style.opacity = 0;
		
		// show poof
		var event = window.event;
		showPoof(event.x, event.y);
		
		setTimeout("removeChannelFromGroup('" + obj.lang + "', '" + parent.id + "')", 500);		// it takes the poof 500ms to play
	}
}

function removeChannelFromGroup(channel_id, group) {
	if(allChannels[channel_id] || availableChannels[channel_id]) {		// need to use 'allChannels' here, too, because we may remove channels we just added from the pool
		allChannels[channel_id] ? allChannels[channel_id].unsetGroup(group) : availableChannels[channel_id].unsetGroup(group);
	}
}




// initiates checking for updated channels.xml
function checkChannelsForUpdates()
{
	FernsehPlugin.checkChannelsForUpdates(LANG + ".lproj/channels.xml", LANG, channelListVersion);
}

// called from the plugin when new channels are available
function newChannelsAvailable(description, path, newVersion)
{
	newChannelListVersion = newVersion;
	newChannelDownloadURL = path;
	showDialog(locStr("Neue Senderdefinition verfügbar:<br />")+description+"<br /><br />"+locStr("Jetzt herunterladen?"), locStr("Laden"), loadNewChannels, locStr("Ignorieren"), ignoreNewChannels);
}

function ignoreNewChannels()
{
	channelListVersion = newChannelListVersion;
	blendDialogOut();
}

function loadNewChannels()
{
	blendDialogOut();
	FernsehPlugin.downloadAndInstallNewChannels(LANG + ".lproj/channels.xml", LANG, newChannelDownloadURL);
}

// called from the plugin after the new channels have been downloaded -- or not
function newChannelsInstalled(success)
{
	if (success == "1")
	{
		channelGroup1 = new Array();
		channelGroup2 = new Array();
		channelsLoaded = false;
		availableChannels = new Object();
		cleanUnusedChannels();
		loadAllChannels(false);
		
		showDialog(locStr("Neue Senderdefinition geladen und installiert"), locStr("OK"), blendDialogOut);
	} else {
		showDialog(locStr("Fehler beim Herunterladen der neuen Senderdefinition"), locStr("OK"), blendDialogOut);
	}
}




// shows/hides the channel source list
var channelSourceListShown = false;
var channelSourceAnimation = { start:0, end:0, duration:300, from:null, to:null }

function toggleChannelSourceList() {
	if(!channelSourceListShown && !channelsAvailable) {
		loadAllChannels('and_toggle');
		return false;
	}
	
	var list = getObj('channelGroupAreaParent');
	var source = getObj('channelSourceAreaParent');
	var message = getObj('showChannelSourceText');
	
	channelSourceAnimation.from = channelSourceAnimation.to ? channelSourceAnimation.to : (channelSourceListShown ? 55 : 100);
	
	// hide
	if(channelSourceListShown) {
		channelSourceAnimation.to = 100;
		message.innerHTML = '';
	}
	
	// show
	else {
		message.innerHTML = locStr('lade Sender...');
		addChannelParentsToSelector('channelParentSourceArea', 'click_list');
		message.innerHTML = '';
		channelSourceAnimation.to = 55;
	}
	
	var now = (new Date()).getTime();
	channelSourceAnimation.start = now;
	channelSourceAnimation.end = now + channelSourceAnimation.duration;
	
	channelSourceListAnimation();
}

// animation
function channelSourceListAnimation() {
	var list = getObj('channelGroupAreaParent');
	var source =getObj('channelSourceAreaParent');
	var now = (new Date()).getTime();
	
	// step
	if(now < channelSourceAnimation.end) {
		var delta = (now - channelSourceAnimation.start) / channelSourceAnimation.duration;
		var factor = ((delta * delta) * 3.0) - ((delta * delta * delta) * 2.0);
		var size = (channelSourceAnimation.from * (1-factor)) + (factor * channelSourceAnimation.to);
		
		list.style.height = size + '%';
		source.style.top = size + '%';
		setTimeout("channelSourceListAnimation()", 10);
	}
	
	// finish
	else {
		list.style.height = channelSourceAnimation.to + '%';
		source.style.top = channelSourceAnimation.to + '%';
		
		var button = getObj('showChannelSource');
		var message = getObj('showChannelSourceText');
		
		// hidden
		if(channelSourceListShown) {
			channelSourceListShown = false;
			button.src = "Images/bigPlus.png";
		}
		
		// shown
		else {
			channelSourceListShown = true;
			button.src = "Images/moveDown.png";
			message.innerHTML = locStr('Sender in Gruppe ziehen');
		}
	}
}



// returns the channel group with the desired ID
function getChannelParentByID(id) {
	if(allChannelParents) {
		for(var i = 0; i < allChannelParents.length; i++) {
			if(id == allChannelParents[i].getID()) {
				return allChannelParents[i];
			}
		}
	}
	return null;
}



// ************************
// the channel-GROUP object
function channelParentObj(xml) {
	if(!xml)
		return null;
	
	if(! this instanceof channelParentObj) {
		return new channelParentObj(xml);
	}
	
	// really a group?
	if('channelparent' != xml.nodeName.toLowerCase()) {
		//alert("channelParentObj(): Not a channelparent element: " + xml.nodeName);
		return null;
	}
	
	
	// Attributes
	this.id = xml.hasAttribute('id') ? xml.getAttribute('id') : null;
	if(!this.id) {
		alert("channelParentObj(): Must have an ID!");
		return null;
	}
	this.name = xml.hasAttribute('name') ? xml.getAttribute('name') : 'unnamed group';
	this.channels = new Array();
	
	
	// Methods
	this.addChannel = function(channel) {
		channel.setParent(this.id);
		this.channels.push(channel);
	}
	
	this.addChannelsFromXML = function(xml) {
		if(xml) {
			var childs = getElementNodes(xml);
			for(var i = 0; i < childs.length; i++) {
				var chan = new channelObj(childs[i]);
				this.addChannel(chan);
			}
		}
	}
	
	this.getID = function() {
		return this.id;
	}
	
	this.getName = function() {
		return this.name;
	}
	
	this.getChannels = function() {
		return this.channels;
	}
	
	
	// add to stack
	this.addChannelsFromXML(xml);
	allChannelParents.push(this);
	return this;
}




// ******************
// the channel object
function channelObj(xml) {
	if(!xml)
		return null;
	
	if(! this instanceof channelObj) {
		return new channelObj(xml);
	}
	
	// channelParent?
	if('channelparent' == xml.nodeName.toLowerCase()) {
		return new channelParentObj(xml);
	}
	
	
	// ID. If it already exists on availableChannels, return this instance.
	this.id = xml.hasAttribute('id') ? xml.getAttribute('id') : '';
	if(availableChannels[this.id]) {
		allChannels[this.id] = availableChannels[this.id];
		return availableChannels[this.id];
	}
	
	// Attributes
	this.complementary_id = xml.hasAttribute('complementary_id') ? xml.getAttribute('complementary_id') : null;
	this.name = xml.hasAttribute('name') ? xml.getAttribute('name') : 'unknown name';
	this.altname = xml.hasAttribute('altname') ? xml.getAttribute('altname').split(/,\s*/) : new Array();
	this.default_for_locale = xml.hasAttribute('default_for_locale') ? xml.getAttribute('default_for_locale').split(/,\s*/) : new Array();
	this.parent = null;
	this.inGroups = new Array();
	this.assignedTuner = null;
	this.div = null;
	
	
	// Methods
	this.setParent = function(parent_id) {
		this.parent = parent_id;
	}
	this.unsetParent = function() {
		this.parent = null;
	}
	
	this.setGroup = function(group) {
		if(group && !this.isInGroup(group)) {
			this.inGroups.push(group);
			if(!availableChannels[this.id]) {
				availableChannels[this.id] = this;
			}
			
			// add to the list within its group if it's not already there
			var parent = getObj(group);
			var nodes = getElementNodes(parent);
			for(var i = 0; i < nodes.length; i++) {
				if(this.id == nodes[i].lang)
					return;
			}
			
			parent.appendChild(this.getDraggableDiv());
		}
	}
	this.unsetGroup = function(group) {
		if(group) {
			var temp = new Array();
			for(var i = 0; i < this.inGroups.length; i++) {
				if(group != this.inGroups[i]) {
					temp.push(this.inGroups[i]);
				}
			}
			this.inGroups = temp;
			if(temp.length < 1) {
				delete availableChannels[this.id];
			}
			
			// remove from the list
			var parent = getObj(group);
			var nodes = getElementNodes(parent);
			for(var i = 0; i < nodes.length; i++) {
				if(this.id == nodes[i].lang) {
					parent.removeChild(nodes[i]);
				}
			}
		}
	}
	this.isInGroup = function(group) {
		if(group) {
			for(var i = 0; i < this.inGroups.length; i++) {
				if(group == this.inGroups[i]) {
					return true;
				}
			}
		}
		return false;
	}
	
	this.getID = function() {
		return this.id;
	}
	
	this.getName = function() {
		return this.name;
	}
	
	this.getGroup = function() {
		return this.group;
	}
	
	this.setTuner = function(tchan) {
		this.assignedTuner = tchan;
	}
	
	this.getTuner = function() {
		return this.assignedTuner ? this.assignedTuner : '';
	}
	
	this.getGuessChannelNames = function() {
		var arr = new Array();
		arr.push(this.name.toLowerCase().replace(/ /g, ""));
		if(this.altname && (this.altname.length > 0)) {
			for(var i = 0; i < this.altname.length; i++) {
				arr.push(this.altname[i]);
			}
		}
		
		return arr;
	}
	
	this.getDraggableDiv = function(force_new) {
		var div = document.createElement("div");
		div.setAttribute("class", "divDrag_item");
		div.setAttribute("lang", this.id);
		
		var img = document.createElement("img");
		img.setAttribute("class", "removeknob");
		img.setAttribute("src", "Images/Cross.png");
		img.setAttribute("alt", "x");
		img.setAttribute("onclick", "animateRemoveChannelFromGroup(this.parentNode)");
		
		var text = document.createElement("span");
		text.innerHTML = this.name;
		text.setAttribute("class", "divDrag_titlespan");
		text.setAttribute("onmousedown", "divDragDown(event)");
		
		div.appendChild(img);
		div.appendChild(text);
		
		this.div = div;
		return div;
	}
	
	this.nameMatches = function(string) {
		if(string) {
			string = string.replace(/\//g, '\/');
			if(this.name.search(eval("/" + string + "/i")) > -1) {
				return true;
			}
			for(var i = 0; i < this.altname.length; i++) {
				if(this.altname[i].search(eval("/" + string + "/i")) > -1) {
					return true;
				}
			}
		}
		return false;
	}
	
	// add to stack
	allChannels[this.id] = this;
	return this;
}
